'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.getProxyContainer = getProxyContainer;
exports.default = getKeyInjector;
var fnPrefix = '@@';
var fnPrefixLength = fnPrefix.length;

function memoize(fn) {
  var value = void 0;
  var executed = false;

  return function () {
    if (!executed) {
      executed = true;
      value = fn.apply(this, arguments);
    }

    return value;
  };
}

function getProxyContainer(fn, stateContainer) {
  return Object.keys(stateContainer).reduce(function (acc, key) {

    var cleanKey = typeof stateContainer[key] === 'function' ? key.substr(fnPrefixLength) : key;

    Object.defineProperty(acc, cleanKey, {
      enumerable: false,
      get: memoize(function () {
        fn.updatedBy[cleanKey] = true;
        return stateContainer[cleanKey];
      })
    });

    return acc;
  }, {});
}

function getKeyInjector(newStateContainer, proxyStateContainer, source, newValues) {
  return function (key) {
    if (typeof source[key] !== 'function') {

      newStateContainer[key] = source[key];
    } else {

      var getterKey = key.startsWith(fnPrefix) ? key.substr(fnPrefixLength) : key;
      var functionKey = fnPrefix + getterKey;

      var getter = memoize(function () {
        var updatedBy = Object.keys(source[key].updatedBy);
        var shouldComputeValue = !newValues || updatedBy.length === 0 || updatedBy.some(function (key) {
          return newValues.hasOwnProperty(key) && newValues[key] !== source[key];
        });

        if (shouldComputeValue) {
          var proxyContainer = getProxyContainer(source[key], newStateContainer);
          return source[key].call(proxyContainer, proxyContainer);
        } else {
          return source[getterKey];
        }
      });

      Object.defineProperty(newStateContainer, functionKey, {
        enumerable: true,
        writable: false,
        value: source[key]
      });

      Object.defineProperty(newStateContainer, getterKey, {
        enumerable: false,
        get: getter
      });
    }
  };
}